﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.TextFormatting;
using FooEditEngine.WPF.Properties;

namespace FooEditEngine.WPF
{
    class PrintableTextRender : IPrintableTextRender
    {
        FontFamily Font;
        double FontSize;
        DrawingContext Context;
        int _TabWidthChar;
        double TabWidth, _LineNumberWidth,_LineHeight;

        public PrintableTextRender(FontFamily font, double fontSize)
        {
            this.Font = font;
            this.FontSize = fontSize;

            TextLayout layout = new TextLayout("0", this.Font, this.FontSize, Brushes.Black, 0);
            this._LineNumberWidth = layout.Lines[0].Width * int.Parse(Resources.LineNumberLength);
            this._LineHeight = layout.Lines[0].Height;
            layout.Dispose();
        }
        
        public Rectangle ClipRect
        {
            get;
            set;
        }

        public double LineNemberWidth
        {
            get { return this._LineNumberWidth; }
        }

        public int TabWidthChar
        {
            get
            {
                return this._TabWidthChar;
            }
            set
            {
                TextLayout layout = new TextLayout("a", this.Font, this.FontSize, Brushes.Black, 0);
                double width = layout.Lines[0].WidthIncludingTrailingWhitespace;
                layout.Dispose();
                this.TabWidth = width * value;
                this._TabWidthChar = value;
            }
        }

        public event ChangedRenderResourceEventHandler ChangedRenderResource;

        public void SetDrawingContext(DrawingContext dc)
        {
            this.Context = dc;
        }

        public void DrawCachedBitmap(Rectangle rect)
        {
            return;
        }

        public void CacheContent()
        {
            return;
        }

        public bool IsVaildCache()
        {
            return false;
        }

        public void DrawLineNumber(int lineNumber, double x, double y)
        {
            string lineNumberFormat = "{0," + Resources.LineNumberLength + "}";
            TextLayout layout = new TextLayout(string.Format(lineNumberFormat, lineNumber), this.Font, this.FontSize, Brushes.Black, 0);
            layout.Draw(this.Context, x, y);
            layout.Dispose();
        }

        public void DrawLine(Point from, Point to)
        {
            return;
        }

        public void DrawCaret(Rectangle rect)
        {
            return;
        }

        public void FillBackground(Rectangle rect)
        {
            return;
        }

        public void DrawOneLine(LineToIndexTable lti, int row, double x, double y, IEnumerable<Selection> SelectRanges, IEnumerable<Marker> MarkerRanges)
        {
            TextLayout layout = new TextLayout(lti[row], this.Font, this.FontSize, Brushes.Black, 0);
            layout.Draw(this.Context, x, y);
            layout.Dispose();
        }

        public int GetIndexFromX(string str, double x)
        {
            return 0;
        }

        public double GetWidthFromIndex(string str, int index)
        {
            return 0;
        }

        public double GetWidth(string str)
        {
            return 0;
        }

        public double GetXFromIndex(string str, int index)
        {
            return 0;
        }

        public double GetHeight(string str)
        {
            return this._LineHeight;
        }

        public int AlignIndexToNearestCluster(string str, int index, AlignDirection flow)
        {
            return 0;
        }

        public List<LineToIndexTableData> BreakLine(Document doc, int startIndex, int endIndex, double wrapwidth)
        {
            List<LineToIndexTableData> output = new List<LineToIndexTableData>();

            foreach (string str in Util.GetLines(doc, startIndex, endIndex))
            {
                TextLayout layout = new TextLayout(str, this.Font, this.FontSize,Brushes.Black, wrapwidth);

                int i = startIndex;
                foreach (TextLine line in layout.Lines)
                {
                    if (line.Length == 0 && line.NewlineLength == 0)
                        continue;

                    int length = line.Length;

                    IList<TextSpan<TextRun>> runs = line.GetTextRunSpans();
                    if (runs.Last().Value is TextEndOfParagraph)
                        length--;

                    bool lineend = false;
                    if (line.NewlineLength > 0)
                        lineend = true;

                    output.Add(new LineToIndexTableData(i, length, lineend, null));
                    i += line.Length;
                }

                layout.Dispose();

                startIndex += str.Length;
            }

            if (output.Count > 0)
                output.Last().lineend = true;

            return output;
        }

        public void DrawString(string str, double x, double y, StringAlignment align)
        {
            TextAlignment TextAlign = TextAlignment.Left;
            switch (align)
            {
                case StringAlignment.Left:
                    TextAlign = TextAlignment.Left;
                    break;
                case StringAlignment.Center:
                    TextAlign = TextAlignment.Center;
                    break;
                case StringAlignment.Right:
                    TextAlign = TextAlignment.Right;
                    break;
            }
            TextLayout layout = new TextLayout(str, this.Font, this.FontSize, Brushes.Black, this.ClipRect.Width,TextAlign);
            layout.Draw(this.Context, x, y);
            layout.Dispose();
        }

        public float HeaderHeight
        {
            get { return (float)this.GetHeight(""); }
        }

        public float FooterHeight
        {
            get { return (float)this.GetHeight(""); }
        }
    }
}
